// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "Apl.h"

#include "MainFrm.h"
#include "ParentView.h"
#include "AplView.h"
#include "AplDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
   //{{AFX_MSG_MAP(CMainFrame)
   ON_WM_CREATE()
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

static UINT indicators[] =
{
   ID_SEPARATOR,           // status line indicator
   ID_INDICATOR_CAPS,
   ID_INDICATOR_NUM,
   ID_INDICATOR_SCRL,
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
   m_pPositionHelper = NULL;
}

CMainFrame::~CMainFrame()
{
   delete m_pPositionHelper;
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
   if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
      return -1;
   
   if (!m_wndToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
      | CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
      !m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
   {
      TRACE0("Failed to create toolbar\n");
      return -1;      // fail to create
   }

   if (!m_wndStatusBar.Create(this) ||
      !m_wndStatusBar.SetIndicators(indicators,
        sizeof(indicators)/sizeof(UINT)))
   {
      TRACE0("Failed to create status bar\n");
      return -1;      // fail to create
   }

   // TODO: Delete these three lines if you don't want the toolbar to
   //  be dockable
   m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
   EnableDocking(CBRS_ALIGN_ANY);
   DockControlBar(&m_wndToolBar);

// create the save/load position helper
   m_pPositionHelper = 
      new CWndSaveRestoreHelper(m_hWnd, _T("Settings"), _T("MainWindowPosition"));
   
   if (m_pPositionHelper == NULL)
      return -1;

// creation is successful
   return 0;
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame drawing and redrawing

BOOL CMainFrame::OnCreateClient(LPCREATESTRUCT /*lpcs*/,
   CCreateContext* pContext)
{
//
// this is the desired views layout 
//
//  |-----------------|---------------------------------|
//  |   Parent View   |     Process View (CAplView)     |
//  |  (CParentView)  |---------------------------------|
//  |                 |     Module View  (CModuleView)  |
//  |-----------------|---------------------------------|
//

// create a splitter with 1 row, 2 columns
   if (!m_wndSplitter.CreateStatic(this, 1, 2))
   {
      TRACE("Failed to CreateStaticSplitter");
      return FALSE;
   }

// in fact, it is not possible to know the exact dimension of the client area 
// available for the splitter since the toolbar/status bar are created AFTERWARDS
   CRect FrameRect;
   GetClientRect(FrameRect);

// add the left Parent tree view pane
   if (!m_wndSplitter.CreateView(
         0, 0, 
         RUNTIME_CLASS(CParentView), 
         CSize(FrameRect.Width() / 3, 0),
         pContext
         )
      )
   {
      TRACE("Failed to create left pane");
      return FALSE;
   }
// add the second splitter pane - which is a nested splitter with 2 rows
   if (!m_wndSubSplitter.CreateStatic(
         &m_wndSplitter,     // our parent window is the first splitter
         2, 1,               // the new splitter has 2 rows, 1 column
         WS_CHILD | WS_VISIBLE | WS_BORDER,  // style, WS_BORDER is needed
         m_wndSplitter.IdFromRowCol(0, 1)
            // inner splitter is in the first row, second column of outer splitter
         )
      )
   {
      TRACE("Failed to create nested splitter");
      return FALSE;
   }

// now create the two views inside the nested splitter
//
// add the upper splitter pane which is supposed to take 1/2 of the client area
   if (!m_wndSubSplitter.CreateView(
         0, 0,
         RUNTIME_CLASS(CAplView),
         CSize(0, FrameRect.Height() / 2), 
         pContext
         )
      )
   {
      TRACE("Failed to create upper pane\n");
      return FALSE;
   }

// add the lower splitter pane which will use the remaining client area
   if (!m_wndSubSplitter.CreateView(
         1, 0,
         RUNTIME_CLASS(CModuleView),
         CSize(0, 0),
         pContext
         )
      )
   {
      TRACE("Failed to create lower pane\n");
      return FALSE;
   }

   // Once views are created, it is time to set their font
   if (pContext != NULL)
   {
      CAplDoc* pDocument = (CAplDoc*)pContext->m_pCurrentDoc;
      pDocument->NotifyDefaultFont();
   }
   else
      TRACE("Impossible to get frame creation parameters");

// don't call the default MFC processing since the views are already created
// return CFrameWnd::OnCreateClient(lpcs, pContext);
   return TRUE;
}


BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
// change the default style to remove the document name from the title
   cs.style &= ~FWS_ADDTOTITLE;
   
// default MFC processing
   return CFrameWnd::PreCreateWindow(cs);
}

void CMainFrame::RestoreFramePosition()
{
   if (m_pPositionHelper != NULL)
   {
      if (!m_pPositionHelper->Restore())
         CenterWindow();
      
      if (!IsWindowVisible())
         ShowWindow(SW_SHOW);
   }
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
   CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
   CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

// don't forget to save the position and size of the frame
// before it is destroyed
void CMainFrame::OnDestroy() 
{
// save the frame position
   if (m_pPositionHelper != NULL)
      m_pPositionHelper->Save();

// default processing
   CFrameWnd::OnDestroy();
}
