// ModuleView.cpp : implementation file
//

#include "stdafx.h"
#include "Apl.h"
#include "ModuleView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CModuleView

IMPLEMENT_DYNCREATE(CModuleView, CListView)

CModuleView::CModuleView()
{
}

CModuleView::~CModuleView()
{
}


BEGIN_MESSAGE_MAP(CModuleView, CListView)
   //{{AFX_MSG_MAP(CModuleView)
	ON_WM_CREATE()
	ON_NOTIFY_REFLECT(LVN_GETDISPINFO, OnGetDispInfo)
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	ON_UPDATE_COMMAND_UI(ID_EDIT_COPY, OnUpdateEditCopy)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CModuleView drawing


/////////////////////////////////////////////////////////////////////////////
// CModuleView diagnostics

#ifdef _DEBUG
void CModuleView::AssertValid() const
{
   CListView::AssertValid();
}

void CModuleView::Dump(CDumpContext& dc) const
{
   CListView::Dump(dc);
}
#endif //_DEBUG

#ifdef _DEBUG  // debug version of ModuleView.cpp
CAplDoc* CModuleView::GetDocument() // non-debug version is inline
{
   ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CAplDoc)));
   return (CAplDoc*)m_pDocument;
}
#endif //_DEBUG
/////////////////////////////////////////////////////////////////////////////
// CModuleView message handlers

BOOL CModuleView::PreCreateWindow(CREATESTRUCT& cs)
{
// default processing
   BOOL bReturn = CListView::PreCreateWindow(cs);

// the list is on REPORT mode by default
   cs.style |= LVS_REPORT;

// the list is single selection only
   cs.style |= LVS_SINGLESEL;

// the selected item stays in the selected color even 
// if the list view loses focus
   cs.style |= LVS_SHOWSELALWAYS;

   return bReturn;
}

// handler for WM_CREATE message which is used to define the columns of the list
int CModuleView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
// default MFC processing
   if (CListView::OnCreate(lpCreateStruct) == -1)
      return -1;

// get a reference to the mapped CListCtrl   
   CListCtrl& Itself = GetListCtrl();

// set the columns for the header
   LVCOLUMN Column;
   Column.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;

// Add the columns.
   Column.cx = 250;
   Column.iSubItem = COLUMN_MODULENAME;
   Column.pszText = _T("Name");
   Column.fmt = LVCFMT_LEFT;
   Itself.InsertColumn(Column.iSubItem, &Column);

   Column.cx = 100;
   Column.iSubItem = COLUMN_ADDRESS;
   Column.pszText = _T("Address");
   Column.fmt = LVCFMT_RIGHT;
   Itself.InsertColumn(Column.iSubItem, &Column);

// if creation is successful
   return 0;
}

void CModuleView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
// lHint = View Message
   switch(lHint)
   {
   // default MFC redrawing process
      case VM_REDRAW :
         CListView::OnUpdate(pSender, lHint, pHint);
      break;

   // the user has selected a new process
      case VM_SET_CURRENT_PROCESS :
      {
         CProcess* pProcess = (CProcess*)pHint;
         FillModuleList(pProcess);
      }
      break;

   // the current font has been changed by the user
      case VM_CHANGEFONT:
      {
      // change list control font
         SendMessage(WM_SETFONT, (WPARAM)(HFONT)pHint, 0L); 

      // redraw all
         Invalidate(TRUE);
      }
      break;

   }
}

// fill the list view with the modules associated to the given process
void CModuleView::FillModuleList(CProcess* pProcess)
{
// don't forget to delete the actual content of the list view
   GetListCtrl().DeleteAllItems();

// fill it according to the given process module list
   if (pProcess != NULL)
   {
      CModuleList& ModuleList = pProcess->GetModuleList();

      int iCount = 0;
      int Pos = 0;
      CModule* pModule = NULL;
      for (
            pModule = ModuleList.GetFirst(Pos); 
            (pModule != NULL); 
            pModule = ModuleList.GetNext(Pos)
          )
      {
      // add each module into the list
         if (pModule != NULL)
            InsertItem(pModule, iCount++);
      }

   // set the selection on the first module
      if (iCount > 0)
         GetListCtrl().SetItemState(
            0, 
            LVIS_SELECTED | LVIS_FOCUSED, 
            LVIS_SELECTED | LVIS_FOCUSED
         );
   }
   else
   {
   // "no module info" 
      LVITEM item;
      item.iItem     = 0;
      item.iSubItem  = 0;
      item.mask      = LVIF_TEXT;
      item.pszText   = _T("No Module Info...");

      GetListCtrl().InsertItem (&item);
   }
}

int CModuleView::InsertItem(LPVOID pItem, int nItem)
{
   LVITEM item;
   item.iItem     = (nItem == -1)? GetListCtrl().GetItemCount() : nItem;
   item.iSubItem  = 0;
   item.mask      = LVIF_TEXT | LVIF_PARAM;
   item.pszText   = LPSTR_TEXTCALLBACK;
   item.lParam    = (LPARAM) pItem;

   return GetListCtrl().InsertItem(&item);
}

void CModuleView::OnGetDispInfo(NMHDR* pNMHDR, LRESULT* pResult) 
{
   LV_DISPINFO* pDispInfo = (LV_DISPINFO*)pNMHDR;

// call virtual method OnNeedText() to get the text to be displayed in each column
   if (pDispInfo->item.mask & LVIF_TEXT)
   {
      LPVOID pItem = (LPVOID) pDispInfo->item.lParam;
      CString  szLabel;
      OnNeedText (pItem, pDispInfo->item.iSubItem, szLabel);
      ::lstrcpy(pDispInfo->item.pszText, szLabel);
   }
      
   *pResult = 0;
}

void CModuleView::OnNeedText(LPVOID pItem, int nColumn, CString& szText)
{
   szText = _T("");

   CModule* pObject = (CModule*)pItem;

   if (pObject != NULL)
      switch (nColumn)
      {
      // address
         case COLUMN_ADDRESS:
            szText.Format(_T("%x"), pObject->GetModuleHandle());
         break;

      // name
         case COLUMN_MODULENAME:
            szText = pObject->GetModuleName();
         break;
      }
}

void CModuleView::OnEditCopy() 
{
// get the string to be sent to the clipboard
   LPTSTR pText = GetClipboardString();
   if (pText != NULL)
   {
      CAplApp* pApp = (CAplApp*)AfxGetApp();
      if (!pApp->CopyTextToClipboard(pText))
         ::MessageBox(
            ::GetActiveWindow(),
            _T("Impossible to copy the module list to the clipboard!"),
            _T("Advanced Process List"),
            MB_OK | MB_ICONHAND
            );
   // don't forget to delete the string
      delete [] pText;
   }
   else
      ::MessageBox(
         ::GetActiveWindow(),
         _T("Nothing to copy to clipboard..."),
         _T("Advanced Process List"),
         MB_OK | MB_ICONINFORMATION
         );
}

void CModuleView::OnUpdateEditCopy(CCmdUI* pCmdUI) 
{
// possible only if there is a process selected or something to copy
   CAplDoc* pDoc = GetDocument();
   ASSERT(pDoc != NULL);

   if (pDoc != NULL)
      pCmdUI->Enable(
         (pDoc->GetSelectedProcess() != NULL) &&
         (GetListCtrl().GetItemCount() > 0)
      );
}

LPTSTR CModuleView::GetClipboardString()
{
   TCHAR* pText = NULL;

// fail if there is no selected process
   CAplDoc* pDoc = GetDocument();
   ASSERT(pDoc != NULL);
   if (pDoc != NULL)
   {
      CProcess* pProcess = pDoc->GetSelectedProcess();
      if (pProcess != NULL)
      {
      // concat each module description
         CModuleList& ModuleList = pProcess->GetModuleList();

      // allocate the returned buffer max len of last column
      // according to the max length of each column
         pText = new TCHAR[(ModuleList.GetCount() + 1) * (_MAX_PATH + 16)];
         if (pText != NULL)
         {
            CString  szLine;
            CString  szBuffer;
            TCHAR*   pNext    = pText;
            int      iCount   = 0;
            int      Pos      = 0;
            CModule* pModule  = NULL;

         // title 
            szLine.Format("Module\tAddress\r\n");
            _tcscpy(pNext, szLine);
            pNext += szLine.GetLength();

         // each module description
            for (
                  pModule = ModuleList.GetFirst(Pos); 
                  (pModule != NULL); 
                  pModule = ModuleList.GetNext(Pos)
                )
            {
               if (pModule != NULL)
               {
               // add each module into the buffer
                  szBuffer = pModule->GetModuleName();
                  szLine.Format("%s\t%x\r\n", szBuffer, pModule->GetModuleHandle());
                  _tcscpy(pNext, szLine);
                  pNext += szLine.GetLength();
               }
            }
         }
      }
   }

   return pText;
}
