// Process.cpp: implementation of the CProcess class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Process.h"
#include "NtQuery.h"
#include "Psapi.h"

#pragma comment(lib, "psapi.lib")
#pragma comment(lib, "ntdll.lib")
#pragma message("  CProcess and CProcessList require psapi.lib and ntdll.lib to link")

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

IMPLEMENT_DYNCREATE(CProcess, CObject)

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CProcess::CProcess()
{
   DefaultInit();
}

CProcess::~CProcess()
{

}

BOOL CProcess::AttachProcess(DWORD PID)
{
// check for given parameter
   if (PID <= 0)
   {
   // it should be the "System Idle Process"
      m_szName = _T("System Idle Process");
   }

// check if we can get information for this process 
   HANDLE hProcess = ::OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, PID);

   if (hProcess != NULL)
   {
      TCHAR szBuffer[_MAX_PATH+1];
   // set the process ID
      m_dwPId = PID;

   // get the module name and filename 
      if (::GetModuleBaseName(hProcess, NULL, szBuffer, _MAX_PATH) == 0)
         m_szName = _T("System");
      else
      {
         m_szName = szBuffer;

      // remove the ".exe" from the module name
         m_szName = m_szName.Left(m_szName.ReverseFind(_T('.')));
      }
      
      if (::GetModuleFileNameEx(hProcess, NULL, szBuffer, _MAX_PATH) == 0)
         m_szFilename = _T("?");
      else
         m_szFilename = szBuffer;

   // search for the main window
      m_hMainWnd = GetMainWindow(m_dwPId);
      if (::IsWindow(m_hMainWnd))
      {
         ::GetWindowText(m_hMainWnd, m_szMainWndTitle.GetBuffer(_MAX_PATH), _MAX_PATH);
         m_szMainWndTitle.ReleaseBuffer(-1);
      }
      else
         m_szMainWndTitle = _T("....no window");

   // get the memory info
      PROCESS_MEMORY_COUNTERS pmc;
      pmc.cb = sizeof(PROCESS_MEMORY_COUNTERS);
      if (::GetProcessMemoryInfo(hProcess, &pmc, sizeof(PROCESS_MEMORY_COUNTERS)))
         m_WorkingSet = pmc.WorkingSetSize;
      else
      {
         TRACE("Impossible to get process(%u) working set...\n", PID);
      }

   // get its parent process ID
      PROCESS_BASIC_INFORMATION  pbi;
      DWORD                      dwSize;
      int                        iReturn;
      pbi.InheritedFromUniqueProcessId = UNKNOWN_PROCESS_ID;
      iReturn = 
         ::NtQueryInformationProcess(
            hProcess, 
            ProcessBasicInformation, 
            &pbi, 
            sizeof(pbi), 
            &dwSize
            );

   // NtQueryInformationProcess returns a negative value if it fails
      if (iReturn >= 0) 
         m_ParentProcessID = pbi.InheritedFromUniqueProcessId;
      else
      {
         TRACE("Impossible to get process(%u) parent process ID...\n", PID);
      }

   // query for handle count
      DWORD dwHandleCount;
      iReturn = 
         ::NtQueryInformationProcess(
            hProcess, 
            ProcessHandleCount, 
            &dwHandleCount, 
            sizeof(dwHandleCount), 
            &dwSize
            );
      if (iReturn >= 0)
         m_HandleCount = dwHandleCount;
      else
      {
         TRACE("Impossible to get process(%u) handles count...\n", PID);
      }

   // fill the module list 
      m_ModuleList.Refresh(hProcess);

   // don't forget to release the process handle
      ::CloseHandle(hProcess);

      return TRUE;
   }
   else
      return FALSE;
}

// set members to default values
BOOL CProcess::DefaultInit()
{
   m_dwPId           = UNKNOWN_PROCESS_ID;
   m_szName          = _T("");
   m_szFilename      = _T("");
   m_szMainWndTitle  = _T("");
   m_hMainWnd        = NULL;
   m_ParentProcessID = 0;
   m_HandleCount     = 0;
   m_WorkingSet      = 0;

   return TRUE;
}

void CProcess::AddChild(CProcess* pProcess)
{
// impossible to add a NULL process
   ASSERT(pProcess != NULL);

// add it into the children array
   if (pProcess != NULL)
      m_ChildProcesses.Add(pProcess);
}

int CProcess::GetChildrenCount()
{
   return m_ChildProcesses.GetSize();
}

//debug assertion files
#ifdef _DEBUG
void CProcess::AssertValid() const
{
   CObject::AssertValid();

   ASSERT(m_dwPId > 0);
}

void CProcess::Dump(CDumpContext& dc) const
{
   CObject::Dump(dc);

   dc << "\nm_szName          = " << (LPCTSTR)m_szName;
   dc << "\nm_dwPId           = " << m_dwPId;
   dc << "\nm_szFilename      = " << (LPCTSTR)m_szFilename;
   dc << "\nm_hMainWnd        = " << m_hMainWnd;
   dc << "\nm_szMainWndTitle  = " << (LPCTSTR)m_szMainWndTitle;
   dc << "\nm_ParentProcessID = " << m_ParentProcessID;
   dc << "\nm_HandleCount     = " << m_HandleCount;
   dc << "\nm_WorkingSet      = " << m_WorkingSet;
   dc << "\n";
}
#endif //_DEBUG

// define windows enumeration structure  
typedef struct tagENUMINFO
{
// In Parameters
   DWORD PId;                // we are looking for the main window of this process

// Out Parameters
   HWND  hWnd;               // THE visible window we are looking for
   HWND  hEmptyWnd;          // visible window with no title
   HWND  hInvisibleWnd;      // invisible window
   HWND  hEmptyInvisibleWnd; // invisible window with no title
} ENUMINFO, *PENUMINFO;   

//get window handle sequence

BOOL CALLBACK EnumWindowsProc(HWND hWnd, LPARAM lParam)
{
   DWORD       pid = 0;
   PENUMINFO   pInfo = (PENUMINFO)lParam;
   TCHAR       szTitle[_MAX_PATH+1];

// sanity checks
   if (pInfo == NULL)
   // stop the enumeration if invalid parameter is given
      return FALSE;

// get the process ID for this window
   if (!::GetWindowThreadProcessId(hWnd, &pid)) 
   // this should never occur :-)
      return TRUE;

// compare the process ID with the one given as search parameter
   if (pInfo->PId == pid) 
   {
   // look for the visibility first
      if (::IsWindowVisible(hWnd)) 
      {
      // next, look for the title next
         if (::GetWindowText(hWnd, szTitle, _MAX_PATH) != 0)
         {
            pInfo->hWnd = hWnd;

         // we have found the right window
            return FALSE;
         }
         else  // we have found a visible window without title
            pInfo->hEmptyWnd = hWnd;
      }
      else  // the window is invisible
      {
      // next, look for the title
         if (::GetWindowText(hWnd, szTitle, _MAX_PATH) != 0)
         // invisible window with a title
            pInfo->hInvisibleWnd = hWnd;  
         else
         // invisible window with no title
            pInfo->hEmptyInvisibleWnd = hWnd; 
      }
   }

// continue the enumeration
   return TRUE;
}

HWND GetMainWindow(DWORD PId)
{
   ENUMINFO EnumInfo;

// set the search parameters 
   EnumInfo.PId = PId;

// set the return parameters to default values
   EnumInfo.hWnd               = NULL;
   EnumInfo.hEmptyWnd          = NULL;
   EnumInfo.hInvisibleWnd      = NULL;
   EnumInfo.hEmptyInvisibleWnd = NULL;

// do the search among the top level windows
   ::EnumWindows((WNDENUMPROC)EnumWindowsProc, (LPARAM)&EnumInfo);

// return the found window if any
   if (EnumInfo.hWnd != NULL)
      return EnumInfo.hWnd;
   else if (EnumInfo.hEmptyWnd != NULL)
      return EnumInfo.hEmptyWnd;
   else if (EnumInfo.hInvisibleWnd != NULL)
      return EnumInfo.hInvisibleWnd;
   else
      return EnumInfo.hEmptyInvisibleWnd;
}


//////////////////////////////////////////////////////////////////////
// CProcessList Class
//////////////////////////////////////////////////////////////////////

IMPLEMENT_DYNCREATE(CProcessList, CObject)

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CProcessList::CProcessList()
{

}

CProcessList::~CProcessList()
{
// empty the map and free each of its element
   DefaultReset();
}

// retrieve and fill the processes list from the system
void CProcessList::Refresh()
{
// 1) don't forget to reset and free the current list
   DefaultReset();

// store the current process list
   DWORD       aProcesses[MAX_PROCESS];
   DWORD       cbNeeded = 0;

// get a snapshot of the processes
   if (!EnumProcesses(aProcesses, sizeof(aProcesses), &cbNeeded))
      return;
    
// Calculate how many process IDs were returned
   DWORD cProcesses = cbNeeded / sizeof(DWORD);

// attach a CProcess object to each process ID
   DWORD     dwProcessID;
   CProcess* pProcess;
   for (DWORD dwCurrentProcess = 0; dwCurrentProcess < cProcesses; dwCurrentProcess++)
   {
      dwProcessID = aProcesses[dwCurrentProcess];

   // 2) add the process definition into the map
      pProcess = new CProcess;
      if (pProcess != NULL)
      {
      // retrieve process information for the current process ID
         if (!pProcess->AttachProcess(aProcesses[dwCurrentProcess]))
            delete pProcess;
         else
         // store into the map
            m_ProcessMap[(LPVOID)dwProcessID] = pProcess;
      }
   }

// 3) a second iteration is needed to fill the children process information
   SetChildrenList();
}

void CProcessList::DefaultReset()
{
// do nothing if the map is empty
   if (m_ProcessMap.IsEmpty())
      return;

// get each element of the map and delete it
   POSITION pos;
   DWORD    key;

// delete one process after the other
   CProcess*   pProcess;

   for (pos = m_ProcessMap.GetStartPosition(); pos != NULL; )
   {
      m_ProcessMap.GetNextAssoc(pos, key, pProcess);
      delete pProcess;
   }

// delete the map itself
   m_ProcessMap.RemoveAll();
}

void CProcessList::SetChildrenList()
{
   CProcess* pParent  = NULL;
   CProcess* pProcess = NULL;

   POSITION Pos = 0;
   DWORD    PID = 0;
   for (pProcess = GetFirst(Pos); (pProcess != NULL); pProcess = GetNext(Pos))
   {
   // get a process pointer to its parent and 
   // add itself into this parent process child list
      if (pProcess != NULL)
      {
         PID = pProcess->GetParentProcessID();
         if (m_ProcessMap.Lookup((LPVOID)PID, pParent))
            pParent->AddChild(pProcess);
      }
   }
}

// return the process info associated to the given process ID
CProcess* CProcessList::GetProcess(DWORD PID)
{
   CProcess* pProcess = NULL;
   if (m_ProcessMap.Lookup((LPVOID)PID, pProcess))
      return pProcess;
   else
      return NULL;
}

// return number of processes in the list
int CProcessList::GetCount()
{
   return m_ProcessMap.GetCount();
}

// return the first process in the list
CProcess* CProcessList::GetFirst(POSITION& pos)
{
   pos = m_ProcessMap.GetStartPosition();
   return GetNext(pos);
}

// next iteration step
CProcess* CProcessList::GetNext(POSITION& pos)
{
   if (pos != NULL)
   {
      DWORD PID = 0;
      CProcess* pProcess = NULL;
      m_ProcessMap.GetNextAssoc(pos, PID, pProcess);
      return pProcess;
   }
   else
      return NULL;
}

#ifdef _DEBUG
void CProcessList::AssertValid() const
{
   CObject::AssertValid();
}

void CProcessList::Dump(CDumpContext& dc) const
{
   CObject::Dump(dc);

   dc << "\n# of modules= " << m_ProcessMap.GetCount();
   dc << "\n";
}
#endif //_DEBUG

