// Process.h: interface for the CProcess class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_PROCESS_H__F923BB61_6CF0_11D2_A0A6_00A0C94CB8B3__INCLUDED_)
#define AFX_PROCESS_H__F923BB61_6CF0_11D2_A0A6_00A0C94CB8B3__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <AfxTempl.h> // needed for template m_ProcessMap
#include "Module.h"

static const int UNKNOWN_PROCESS_ID = ((DWORD)-1);
static const int IDLE_PROCESS_ID    = ((DWORD)-2);
static const int MAX_PROCESS        = 1024;

// get window handle sequence
   BOOL CALLBACK EnumWindowsProc(HWND hWnd, DWORD lParam);
   HWND GetMainWindow(DWORD PId);

class CProcess : public CObject  
{
   DECLARE_DYNCREATE(CProcess)

// constructor/destructor
public:
   CProcess();
   virtual ~CProcess();

// public interface
public:
   const DWORD    GetPID() const      { return m_dwPId; }
   const CString& GetName() const     { return m_szName; }
   const CString& GetFilename() const { return m_szFilename; }
   const CString& GetMainWindowTitle() const  { return m_szMainWndTitle; }
   const HWND     GetMainWindowHandle() const { return m_hMainWnd; }
   const DWORD    GetParentProcessID() const  { return m_ParentProcessID; }
   const DWORD    GetHandleCount() const  { return m_HandleCount; }
   const DWORD    GetWorkingSet() const   { return m_WorkingSet; }

// give access to modules loaded by process through a CModuleList object
   CModuleList& GetModuleList() { return m_ModuleList; }

// provide methods to enumerate spawned processes 
   int GetChildrenCount();

// internal helpers
private:
   BOOL AttachProcess(DWORD PID);
   BOOL DefaultInit();
   void AddChild(CProcess* pProcess);   // used by CProcessList

// internal members
protected:
   DWORD m_dwPId;                      // process ID
   CString  m_szName;                  // name
   CString  m_szFilename;              // filename
   HWND  m_hMainWnd;                   // Window title handle
   CString  m_szMainWndTitle;          // Window title title
   DWORD m_ParentProcessID;            // parent process ID
   DWORD m_HandleCount;                // handles count
   DWORD m_WorkingSet;                 // working set (used physical RAM)
   CModuleList m_ModuleList;           // module list
   CObArray m_ChildProcesses;          // child process list

// debug helpers
public:
#ifdef _DEBUG
   virtual void AssertValid() const;
   virtual void Dump(CDumpContext& dc) const;
#endif

   friend class CProcessList;
};

//CProcessList declaration
class CProcessList : public CObject  
{
DECLARE_DYNCREATE(CProcessList)

// constructor/destructor
public:
   CProcessList();
   virtual ~CProcessList();

// public interface
public:
   void Refresh();
   CProcess* GetProcess(DWORD PID);
   int GetCount();
   CProcess* GetFirst(POSITION& pos);
   CProcess* GetNext(POSITION& pos);

// debug helpers
public:
#ifdef _DEBUG
   virtual void AssertValid() const;
   virtual void Dump(CDumpContext& dc) const;
#endif

// internal helpers
protected:
   void DefaultReset();
   void SetChildrenList();

// internal members
protected:
// store each process description (Ptr = 32 bits --> ID) 
// indexed by its process ID (Ptr = 32 bits --> CProcess*) 
   CTypedPtrMap<CMapPtrToPtr, DWORD, CProcess*> m_ProcessMap;
};


#endif // !defined(AFX_PROCESS_H__F923BB61_6CF0_11D2_A0A6_00A0C94CB8B3__INCLUDED_)
